@@ -16,7 +16,6 @@ import java.awt.Dimension;
 import java.awt.geom.Rectangle2D;
 import java.nio.FloatBuffer;
 import java.text.DecimalFormat;
-import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.LinkedList;
 import java.util.List;
@@ -79,6 +78,48 @@ public class RulerDrawer {
         return oneShotRulerDrawer.drawWithResults(drawingTools, model);
     }
 
+    /**
+     * Draw the ruler
+     * @param drawingTools the {@link DrawingTools} of the canvas where the ruler will be drawn.
+     */
+    public void draw(DrawingTools drawingTools) {
+        oneShotRulerDrawer.draw(drawingTools);
+    }
+
+    /**
+     * Get the model
+     * @return the ruler model.
+     */
+    public RulerModel getModel() {
+        return oneShotRulerDrawer.rulerModel;
+    }
+
+    /**
+     * Get the subticks values
+     * @return the values.
+     */
+    public List<Double> getSubTicksValue() {
+        return oneShotRulerDrawer.subTicksValue;
+    }
+
+    /**
+     * Get the ticks values
+     * @return the values.
+     */
+    public List<Double> getTicksValue() {
+        return oneShotRulerDrawer.ticksValue;
+    }
+
+    /**
+     * Compute different parameters on a ruler
+     * @param drawingTools the {@link DrawingTools} of the canvas where the ruler will be drawn.
+     * @param rulerModel the {@link RulerModel} of the drawn ruler.
+     * @param canvasProjection the canvas projection.
+     */
+    public RulerDrawingResult computeRuler(DrawingTools drawingTools, RulerModel model, Transformation canvasProjection) {
+        return oneShotRulerDrawer.computeRuler(drawingTools, model, canvasProjection);
+    }
+
     /**
      * Set the current {@link RulerSpriteFactory}.
      * All existing sprite will be cleared.
@@ -132,7 +173,7 @@ public class RulerDrawer {
 
         public OneShotRulerDrawer() { }
 
-        public void dispose() {
+        public synchronized void dispose() {
             spriteDimensionMap.clear();
             spritesList.clear();
             if (subTicksValue != null) {
@@ -145,16 +186,51 @@ public class RulerDrawer {
         }
 
         /**
-         * Constructor.
+         * Compute different parameters on a ruler
          * @param drawingTools the {@link DrawingTools} of the canvas where the ruler will be drawn.
          * @param rulerModel the {@link RulerModel} of the drawn ruler.
+         * @param canvasProjection the canvas projection.
          */
-        public RulerDrawingResult drawWithResults(DrawingTools drawingTools, RulerModel rulerModel) {
+        public synchronized RulerDrawingResult computeRuler(DrawingTools drawingTools, RulerModel rulerModel, Transformation canvasProjection) {
+            // Same code as drawWithResults (without drawing)
+            // Historically, computations were made when drawing and they are made before drawing.
+            // TODO: remove drawWithResults ??
+            this.canvasProjection = canvasProjection;
             this.rulerModel = rulerModel;
             subTicksValue = new LinkedList<Double>();
             ticksValue = new LinkedList<Double>();
             spritesList = new LinkedList<PositionedSprite>();
 
+            Vector3d windowTicksDirection = canvasProjection.projectDirection(rulerModel.getTicksDirection());
+            windowTicksDirection = windowTicksDirection.setZ(0);
+
+            Vector3d normalizedProjectedTicksDirection = windowTicksDirection.getNormalized();
+            windowSubTicksDelta = normalizedProjectedTicksDirection.times(rulerModel.getSubTicksLength());
+            windowTicksDelta = normalizedProjectedTicksDirection.times(rulerModel.getTicksLength());
+
+            DecimalFormat format;
+            if (rulerModel.isAutoTicks()) {
+                format = computeAutoGraduation();
+            } else {
+                format = computeUserGraduation();
+            }
+            computeTicksData();
+
+            double distRatio = computeTicksDistanceRatio(windowTicksDirection.getNorm());
+
+            return new RulerDrawingResult(format, ticksValue, subTicksValue, density, distRatio, normalizedProjectedTicksDirection);
+        }
+
+        /**
+         * Constructor.
+         * @param drawingTools the {@link DrawingTools} of the canvas where the ruler will be drawn.
+         * @param rulerModel the {@link RulerModel} of the drawn ruler.
+         */
+        public synchronized RulerDrawingResult drawWithResults(DrawingTools drawingTools, RulerModel rulerModel) {
+            this.rulerModel = rulerModel;
+            subTicksValue = new LinkedList<Double>();
+            ticksValue = new LinkedList<Double>();
+            spritesList = new LinkedList<PositionedSprite>();
             canvasProjection = drawingTools.getTransformationManager().getCanvasProjection();
 
             Vector3d windowTicksDirection = canvasProjection.projectDirection(rulerModel.getTicksDirection());
@@ -171,6 +247,7 @@ public class RulerDrawer {
                 format = computeUserGraduation();
             }
             computeTicksData();
+
             draw(drawingTools);
 
             double distRatio = computeTicksDistanceRatio(windowTicksDirection.getNorm());
@@ -199,7 +276,7 @@ public class RulerDrawer {
          * Actually perform the ruler drawing.
          * @param drawingTools {@link DrawingTools} used to perform the ruler drawing.
          */
-        private void draw(DrawingTools drawingTools) {
+        private synchronized void draw(DrawingTools drawingTools) {
             BuffersManager bufferManager = drawingTools.getCanvas().getBuffersManager();
             ElementsBuffer vertices = bufferManager.createElementsBuffer();
             fillVertices(vertices, rulerModel, ticksValue, subTicksValue, canvasProjection);
@@ -236,7 +313,7 @@ public class RulerDrawer {
             Graduations ticksGraduation = currentGraduations;
             DecimalFormat format = currentGraduations.getFormat();
             boolean canGetMore = true;
-            List<PositionedSprite> newSpritesList = new ArrayList<PositionedSprite>();
+            List<PositionedSprite> newSpritesList = new LinkedList<PositionedSprite>();
             while (currentGraduations != null) {
                 /* The maximum distance to any of the sprites' farthest sides at a given iteration. */
                 double currentMaximalSpritesDistance = 0;
@@ -321,15 +398,14 @@ public class RulerDrawer {
                 while ((subGraduation != null) && (computeTicksDistance(subGraduation) < rulerModel.getMinimalSubTicksDistance())) {
                     subGraduation = subGraduation.getAlternative();
                 }
-
                 if (subGraduation != null) {
                     subTicksValue = subGraduation.getAllValues();
                 } else {
-                    subTicksValue = new ArrayList<Double>();
+                    subTicksValue = new LinkedList<Double>();
                 }
             } else {
-                subTicksValue = new ArrayList<Double>();
-                ticksValue = new ArrayList<Double>();
+                subTicksValue = new LinkedList<Double>();
+                ticksValue = new LinkedList<Double>();
                 density = 0;
             }
         }
@@ -507,22 +583,26 @@ public class RulerDrawer {
             Texture sprite = spriteMap.get(value);
             if (sprite == null) {
                 sprite = spriteFactory.create(value, format, textureManager);
-                spriteMap.put(value, sprite);
+                if (sprite != null) {
+                    spriteMap.put(value, sprite);
+                }
             }
             return sprite;
         }
 
-        private Dimension computeSpriteDimension(double value)
-            {
-                Dimension spriteDimension = spriteDimensionMap.get(value);
-                if (spriteDimension == null) {
-                    spriteDimension = spriteMap.get(value).getDataProvider().getTextureSize();
+        private Dimension computeSpriteDimension(double value) {
+            Dimension spriteDimension = spriteDimensionMap.get(value);
+            if (spriteDimension == null) {
+                Texture sprite = spriteMap.get(value);
+                if (sprite != null) {
+                    spriteDimension = sprite.getDataProvider().getTextureSize();
                     spriteDimensionMap.put(value, spriteDimension);
                 }
-
-                return spriteDimension;
             }
 
+            return spriteDimension;
+        }
+
         /**
          * This class is a basic container for a {@link Texture} and an associated window position.
          *
